@@ -1,7 +1,10 @@
+from __future__ import annotations
+
 import copy
 import operator
 import re
 import warnings
+from typing import TYPE_CHECKING
 
 import erfa
 import numpy as np
@@ -11,6 +14,7 @@ from astropy.constants import c as speed_of_light
 from astropy.table import QTable
 from astropy.time import Time
 from astropy.utils import ShapedLikeNDArray
+from astropy.utils.compat import COPY_IF_NEEDED
 from astropy.utils.data_info import MixinInfo
 from astropy.utils.exceptions import AstropyUserWarning
 
@@ -31,6 +35,9 @@ from .sky_coordinate_parsers import (
     _parse_coordinate_data,
 )
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
 __all__ = ["SkyCoord", "SkyCoordInfo"]
 
 
@@ -288,6 +295,11 @@ class SkyCoord(ShapedLikeNDArray):
     # info property.
     info = SkyCoordInfo()
 
+    # Methods implemented by the underlying frame
+    position_angle: Callable[[BaseCoordinateFrame | SkyCoord], Angle]
+    separation: Callable[[BaseCoordinateFrame | SkyCoord], Angle]
+    separation_3d: Callable[[BaseCoordinateFrame | SkyCoord], Distance]
+
     def __init__(self, *args, copy=True, **kwargs):
         # these are frame attributes set on this SkyCoord but *not* a part of
         # the frame object this SkyCoord contains
@@ -824,12 +836,12 @@ class SkyCoord(ShapedLikeNDArray):
             new_distance = Distance(parallax=starpm[4] << u.arcsec)
 
         icrs2 = ICRS(
-            ra=u.Quantity(starpm[0], u.radian, copy=False),
-            dec=u.Quantity(starpm[1], u.radian, copy=False),
-            pm_ra=u.Quantity(starpm[2], u.radian / u.yr, copy=False),
-            pm_dec=u.Quantity(starpm[3], u.radian / u.yr, copy=False),
+            ra=u.Quantity(starpm[0], u.radian, copy=COPY_IF_NEEDED),
+            dec=u.Quantity(starpm[1], u.radian, copy=COPY_IF_NEEDED),
+            pm_ra=u.Quantity(starpm[2], u.radian / u.yr, copy=COPY_IF_NEEDED),
+            pm_dec=u.Quantity(starpm[3], u.radian / u.yr, copy=COPY_IF_NEEDED),
             distance=new_distance,
-            radial_velocity=u.Quantity(starpm[5], u.km / u.s, copy=False),
+            radial_velocity=u.Quantity(starpm[5], u.km / u.s, copy=COPY_IF_NEEDED),
             differential_type=SphericalDifferential,
         )
 
@@ -1133,111 +1145,6 @@ class SkyCoord(ShapedLikeNDArray):
             )
 
     # High-level convenience methods
-    def separation(self, other):
-        """
-        Computes on-sky separation between this coordinate and another.
-
-        .. note::
-
-            If the ``other`` coordinate object is in a different frame, it is
-            first transformed to the frame of this object. This can lead to
-            unintuitive behavior if not accounted for. Particularly of note is
-            that ``self.separation(other)`` and ``other.separation(self)`` may
-            not give the same answer in this case.
-
-        For more on how to use this (and related) functionality, see the
-        examples in :doc:`astropy:/coordinates/matchsep`.
-
-        Parameters
-        ----------
-        other : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
-            The coordinate to get the separation to.
-
-        Returns
-        -------
-        sep : `~astropy.coordinates.Angle`
-            The on-sky separation between this and the ``other`` coordinate.
-
-        Notes
-        -----
-        The separation is calculated using the Vincenty formula, which
-        is stable at all locations, including poles and antipodes [1]_.
-
-        .. [1] https://en.wikipedia.org/wiki/Great-circle_distance
-
-        """
-        from .angles import Angle, angular_separation
-
-        if not self.is_equivalent_frame(other):
-            try:
-                kwargs = (
-                    {"merge_attributes": False} if isinstance(other, SkyCoord) else {}
-                )
-                other = other.transform_to(self, **kwargs)
-            except TypeError:
-                raise TypeError(
-                    "Can only get separation to another SkyCoord "
-                    "or a coordinate frame with data"
-                )
-
-        lon1 = self.spherical.lon
-        lat1 = self.spherical.lat
-        lon2 = other.spherical.lon
-        lat2 = other.spherical.lat
-
-        # Get the separation as a Quantity, convert to Angle in degrees
-        sep = angular_separation(lon1, lat1, lon2, lat2)
-        return Angle(sep, unit=u.degree)
-
-    def separation_3d(self, other):
-        """
-        Computes three dimensional separation between this coordinate
-        and another.
-
-        For more on how to use this (and related) functionality, see the
-        examples in :doc:`astropy:/coordinates/matchsep`.
-
-        Parameters
-        ----------
-        other : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
-            The coordinate to get the separation to.
-
-        Returns
-        -------
-        sep : `~astropy.coordinates.Distance`
-            The real-space distance between these two coordinates.
-
-        Raises
-        ------
-        ValueError
-            If this or the other coordinate do not have distances.
-        """
-        if not self.is_equivalent_frame(other):
-            try:
-                kwargs = (
-                    {"merge_attributes": False} if isinstance(other, SkyCoord) else {}
-                )
-                other = other.transform_to(self, **kwargs)
-            except TypeError:
-                raise TypeError(
-                    "Can only get separation to another SkyCoord "
-                    "or a coordinate frame with data"
-                )
-
-        if issubclass(self.data.__class__, UnitSphericalRepresentation):
-            raise ValueError(
-                "This object does not have a distance; cannot compute 3d separation."
-            )
-        if issubclass(other.data.__class__, UnitSphericalRepresentation):
-            raise ValueError(
-                "The other object does not have a distance; "
-                "cannot compute 3d separation."
-            )
-
-        c1 = self.cartesian.without_differentials()
-        c2 = other.cartesian.without_differentials()
-        return Distance((c1 - c2).norm())
-
     def spherical_offsets_to(self, tocoord):
         r"""
         Computes angular offsets to go *from* this coordinate *to* another.
@@ -1262,7 +1169,8 @@ class SkyCoord(ShapedLikeNDArray):
         ------
         ValueError
             If the ``tocoord`` is not in the same frame as this one. This is
-            different from the behavior of the `separation`/`separation_3d`
+            different from the behavior of the
+            :meth:`~astropy.coordinates.BaseCoordinateFrame.separation`/:meth:`~astropy.coordinates.BaseCoordinateFrame.separation_3d`
             methods because the offset components depend critically on the
             specific choice of frame.
 
@@ -1274,9 +1182,9 @@ class SkyCoord(ShapedLikeNDArray):
 
         See Also
         --------
-        separation :
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.separation` :
             for the *total* angular offset (not broken out into components).
-        position_angle :
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.position_angle` :
             for the direction of the offset.
 
         """
@@ -1349,7 +1257,7 @@ class SkyCoord(ShapedLikeNDArray):
         -------
         newpoints : `~astropy.coordinates.SkyCoord`
             The coordinates for the location that corresponds to offsetting by
-            the given `position_angle` and `separation`.
+            the given ``position_angle`` and ``separation``.
 
         Notes
         -----
@@ -1366,8 +1274,10 @@ class SkyCoord(ShapedLikeNDArray):
 
         See Also
         --------
-        position_angle : inverse operation for the ``position_angle`` component
-        separation : inverse operation for the ``separation`` component
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.position_angle` :
+            inverse operation for the ``position_angle`` component
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.separation` :
+            inverse operation for the ``separation`` component
 
         """
         from .angles import offset_by
@@ -1522,7 +1432,7 @@ class SkyCoord(ShapedLikeNDArray):
         This is intended for use on `~astropy.coordinates.SkyCoord` objects
         with coordinate arrays, rather than a scalar coordinate.  For a scalar
         coordinate, it is better to use
-        `~astropy.coordinates.SkyCoord.separation`.
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.separation`.
 
         For more on how to use this (and related) functionality, see the
         examples in :doc:`astropy:/coordinates/matchsep`.
@@ -1582,7 +1492,7 @@ class SkyCoord(ShapedLikeNDArray):
         This is intended for use on `~astropy.coordinates.SkyCoord` objects
         with coordinate arrays, rather than a scalar coordinate.  For a scalar
         coordinate, it is better to use
-        `~astropy.coordinates.SkyCoord.separation_3d`.
+        :meth:`~astropy.coordinates.BaseCoordinateFrame.separation_3d`.
 
         For more on how to use this (and related) functionality, see the
         examples in :doc:`astropy:/coordinates/matchsep`.
@@ -1634,53 +1544,6 @@ class SkyCoord(ShapedLikeNDArray):
             searcharoundcoords, self, distlimit, storekdtree="_kdtree_3d"
         )
 
-    def position_angle(self, other):
-        """
-        Computes the on-sky position angle (East of North) between this
-        SkyCoord and another.
-
-        Parameters
-        ----------
-        other : |SkyCoord|
-            The other coordinate to compute the position angle to.  It is
-            treated as the "head" of the vector of the position angle.
-
-        Returns
-        -------
-        pa : `~astropy.coordinates.Angle`
-            The (positive) position angle of the vector pointing from ``self``
-            to ``other``.  If either ``self`` or ``other`` contain arrays, this
-            will be an array following the appropriate `numpy` broadcasting
-            rules.
-
-        Examples
-        --------
-        >>> c1 = SkyCoord(0*u.deg, 0*u.deg)
-        >>> c2 = SkyCoord(1*u.deg, 0*u.deg)
-        >>> c1.position_angle(c2).degree
-        90.0
-        >>> c3 = SkyCoord(1*u.deg, 1*u.deg)
-        >>> c1.position_angle(c3).degree  # doctest: +FLOAT_CMP
-        44.995636455344844
-        """
-        from .angles import position_angle
-
-        if not self.is_equivalent_frame(other):
-            try:
-                other = other.transform_to(self, merge_attributes=False)
-            except TypeError:
-                raise TypeError(
-                    "Can only get position_angle to another "
-                    "SkyCoord or a coordinate frame with data"
-                )
-
-        slat = self.represent_as(UnitSphericalRepresentation).lat
-        slon = self.represent_as(UnitSphericalRepresentation).lon
-        olat = other.represent_as(UnitSphericalRepresentation).lat
-        olon = other.represent_as(UnitSphericalRepresentation).lon
-
-        return position_angle(slon, slat, olon, olat)
-
     def skyoffset_frame(self, rotation=None):
         """
         Returns the sky offset frame with this SkyCoord at the origin.
@@ -1988,27 +1851,22 @@ class SkyCoord(ShapedLikeNDArray):
         elif self.obstime is not None and self.frame.data.differentials:
             # we do need space motion after all
             coo_at_rv_obstime = self.apply_space_motion(obstime)
-        elif self.obstime is None:
-            # warn the user if the object has differentials set
-            if "s" in self.data.differentials:
-                warnings.warn(
-                    "SkyCoord has space motion, and therefore the specified "
-                    "position of the SkyCoord may not be the same as "
-                    "the `obstime` for the radial velocity measurement. "
-                    "This may affect the rv correction at the order of km/s"
-                    "for very high proper motions sources. If you wish to "
-                    "apply space motion of the SkyCoord to correct for this"
-                    "the `obstime` attribute of the SkyCoord must be set",
-                    AstropyUserWarning,
-                )
+        elif self.obstime is None and "s" in self.data.differentials:
+            warnings.warn(
+                "SkyCoord has space motion, and therefore the specified "
+                "position of the SkyCoord may not be the same as "
+                "the `obstime` for the radial velocity measurement. "
+                "This may affect the rv correction at the order of km/s"
+                "for very high proper motions sources. If you wish to "
+                "apply space motion of the SkyCoord to correct for this"
+                "the `obstime` attribute of the SkyCoord must be set",
+                AstropyUserWarning,
+            )
 
-        pos_earth, v_earth = get_body_barycentric_posvel("earth", obstime)
-        if kind == "barycentric":
-            v_origin_to_earth = v_earth
-        elif kind == "heliocentric":
-            v_sun = get_body_barycentric_posvel("sun", obstime)[1]
-            v_origin_to_earth = v_earth - v_sun
-        else:
+        pos_earth, v_origin_to_earth = get_body_barycentric_posvel("earth", obstime)
+        if kind == "heliocentric":
+            v_origin_to_earth -= get_body_barycentric_posvel("sun", obstime)[1]
+        elif kind != "barycentric":
             raise ValueError(
                 "`kind` argument to radial_velocity_correction must "
                 f"be 'barycentric' or 'heliocentric', but got '{kind}'"
@@ -2018,49 +1876,45 @@ class SkyCoord(ShapedLikeNDArray):
         # transforming to GCRS is not the correct thing to do here, since we don't want to
         # include aberration (or light deflection)? Instead, only apply parallax if necessary
         icrs_cart = coo_at_rv_obstime.icrs.cartesian
-        icrs_cart_novel = icrs_cart.without_differentials()
-        if self.data.__class__ is UnitSphericalRepresentation:
-            targcart = icrs_cart_novel
-        else:
-            # skycoord has distances so apply parallax
-            obs_icrs_cart = pos_earth + gcrs_p
-            targcart = icrs_cart_novel - obs_icrs_cart
+        targcart = icrs_cart.without_differentials()
+        if self.data.__class__ is not UnitSphericalRepresentation:
+            # SkyCoord has distances, so apply parallax by calculating
+            # the direction of the target as seen by the observer.
+            targcart -= pos_earth + gcrs_p
             targcart /= targcart.norm()
 
-        if kind == "barycentric":
-            beta_obs = (v_origin_to_earth + gcrs_v) / speed_of_light
-            gamma_obs = 1 / np.sqrt(1 - beta_obs.norm() ** 2)
-            gr = location.gravitational_redshift(obstime)
-            # barycentric redshift according to eq 28 in Wright & Eastmann (2014),
-            # neglecting Shapiro delay and effects of the star's own motion
-            zb = gamma_obs * (1 + beta_obs.dot(targcart)) / (1 + gr / speed_of_light)
-            # try and get terms corresponding to stellar motion.
-            if icrs_cart.differentials:
-                try:
-                    ro = self.icrs.cartesian
-                    beta_star = ro.differentials["s"].to_cartesian() / speed_of_light
-                    # ICRS unit vector at coordinate epoch
-                    ro = ro.without_differentials()
-                    ro /= ro.norm()
-                    zb *= (1 + beta_star.dot(ro)) / (1 + beta_star.dot(targcart))
-                except u.UnitConversionError:
-                    warnings.warn(
-                        "SkyCoord contains some velocity information, but not enough to"
-                        " calculate the full space motion of the source, and so this"
-                        " has been ignored for the purposes of calculating the radial"
-                        " velocity correction. This can lead to errors on the order of"
-                        " metres/second.",
-                        AstropyUserWarning,
-                    )
-
-            zb = zb - 1
-            return zb * speed_of_light
-        else:
-            # do a simpler correction ignoring time dilation and gravitational redshift
-            # this is adequate since Heliocentric corrections shouldn't be used if
-            # cm/s precision is required.
+        if kind == "heliocentric":
+            # Do a simpler correction than for barycentric ignoring time dilation and
+            # gravitational redshift.  This is adequate since heliocentric corrections
+            # shouldn't be used if cm/s precision is required.
             return targcart.dot(v_origin_to_earth + gcrs_v)
 
+        beta_obs = (v_origin_to_earth + gcrs_v) / speed_of_light
+        gamma_obs = 1 / np.sqrt(1 - beta_obs.norm() ** 2)
+        gr = location.gravitational_redshift(obstime)
+        # barycentric redshift according to eq 28 in Wright & Eastmann (2014),
+        # neglecting Shapiro delay and effects of the star's own motion
+        zb = gamma_obs * (1 + beta_obs.dot(targcart)) / (1 + gr / speed_of_light)
+        # try and get terms corresponding to stellar motion.
+        if icrs_cart.differentials:
+            try:
+                ro = self.icrs.cartesian
+                beta_star = ro.differentials["s"].to_cartesian() / speed_of_light
+                # ICRS unit vector at coordinate epoch
+                ro = ro.without_differentials()
+                ro /= ro.norm()
+                zb *= (1 + beta_star.dot(ro)) / (1 + beta_star.dot(targcart))
+            except u.UnitConversionError:
+                warnings.warn(
+                    "SkyCoord contains some velocity information, but not enough to"
+                    " calculate the full space motion of the source, and so this"
+                    " has been ignored for the purposes of calculating the radial"
+                    " velocity correction. This can lead to errors on the order of"
+                    " metres/second.",
+                    AstropyUserWarning,
+                )
+        return (zb - 1) * speed_of_light
+
     # Table interactions
     @classmethod
     def guess_from_table(cls, table, **coord_kwargs):
@@ -2160,8 +2014,7 @@ class SkyCoord(ShapedLikeNDArray):
                     f'Found column "{v.name}" in table, but it was already provided as'
                     ' "{k}" keyword to guess_from_table function.'
                 )
-            else:
-                coord_kwargs[k] = v
+            coord_kwargs[k] = v
 
         return cls(**coord_kwargs)
 
