@@ -3,9 +3,9 @@ A module that provides functions for manipulating bit masks and data quality
 (DQ) arrays.
 
 """
+
 import numbers
 import warnings
-from collections import OrderedDict
 
 import numpy as np
 
@@ -19,8 +19,7 @@ __all__ = [
 
 
 _ENABLE_BITFLAG_CACHING = True
-_MAX_UINT_TYPE = np.maximum_sctype(np.uint)
-_SUPPORTED_FLAGS = int(np.bitwise_not(0, dtype=_MAX_UINT_TYPE, casting="unsafe"))
+_SUPPORTED_FLAGS = int(np.bitwise_not(0, dtype="uint64", casting="unsafe"))
 
 
 def _is_bit_flag(n):
@@ -55,8 +54,6 @@ def _is_int(n):
 class InvalidBitFlag(ValueError):
     """Indicates that a value is not an integer that is a power of 2."""
 
-    pass
-
 
 class BitFlag(int):
     """Bit flags: integer values that are powers of 2."""
@@ -69,8 +66,8 @@ class BitFlag(int):
 
         if not (_is_int(val) and _is_bit_flag(val)):
             raise InvalidBitFlag(
-                "Value '{}' is not a valid bit flag: bit flag value must be "
-                "an integral power of two.".format(val)
+                f"Value '{val}' is not a valid bit flag: bit flag value must be "
+                "an integral power of two."
             )
 
         s = int.__new__(cls, val)
@@ -89,7 +86,7 @@ class BitFlagNameMeta(type):
         attrl = list(map(str.lower, attr))
 
         if _ENABLE_BITFLAG_CACHING:
-            cache = OrderedDict()
+            cache = dict()
 
         for b in bases:
             for k, v in b.__dict__.items():
@@ -176,7 +173,7 @@ class BitFlagNameMeta(type):
             items = dict(items)
 
         return extend_bit_flag_map(
-            cls.__name__ + "_" + "_".join([k for k in items]), cls, **items
+            cls.__name__ + "_" + "_".join(list(items)), cls, **items
         )
 
     def __iadd__(cls, other):
@@ -224,8 +221,6 @@ class BitFlagNameMap(metaclass=BitFlagNameMeta):
 
     """
 
-    pass
-
 
 def extend_bit_flag_map(cls_name, base_cls=BitFlagNameMap, **kwargs):
     """
