@@ -3,22 +3,25 @@
 
 import pickle
 import textwrap
-from collections import OrderedDict
+from itertools import chain, permutations
 
-import pytest
 import numpy as np
+import pytest
 from numpy.testing import assert_array_equal
 
+from astropy import units as u
+from astropy.nddata import NDDataArray
+from astropy.nddata import _testing as nd_testing
 from astropy.nddata.nddata import NDData
 from astropy.nddata.nduncertainty import StdDevUncertainty
-from astropy import units as u
 from astropy.utils import NumpyRNGContext
+from astropy.utils.compat.optional_deps import HAS_DASK
+from astropy.utils.masked import Masked
+from astropy.utils.metadata.tests.test_metadata import MetaBaseTest
 from astropy.wcs import WCS
-from astropy.wcs.wcsapi import HighLevelWCSWrapper, SlicedLowLevelWCS, \
-                               BaseHighLevelWCS
+from astropy.wcs.wcsapi import BaseHighLevelWCS, HighLevelWCSWrapper, SlicedLowLevelWCS
 
 from .test_nduncertainty import FakeUncertainty
-from astropy.nddata import _testing as nd_testing
 
 
 class FakeNumpyArray:
@@ -27,27 +30,29 @@ class FakeNumpyArray:
 
     These attributes are checked for by NDData.
     """
+
     def __init__(self):
         super().__init__()
 
     def shape(self):
         pass
 
-    def __getitem__(self):
+    def __getitem__(self, key):
         pass
 
-    def __array__(self):
+    def __array__(self, dtype=None, copy=None):
         pass
 
     @property
     def dtype(self):
-        return 'fake'
+        return "fake"
 
 
 class MinimalUncertainty:
     """
     Define the minimum attributes acceptable as an uncertainty object.
     """
+
     def __init__(self, value):
         self._uncertainty = value
 
@@ -57,13 +62,21 @@ class MinimalUncertainty:
 
 
 class BadNDDataSubclass(NDData):
-
-    def __init__(self, data, uncertainty=None, mask=None, wcs=None,
-                 meta=None, unit=None):
+    def __init__(
+        self,
+        data,
+        uncertainty=None,
+        mask=None,
+        wcs=None,
+        meta=None,
+        unit=None,
+        psf=None,
+    ):
         self._data = data
         self._uncertainty = uncertainty
         self._mask = mask
         self._wcs = wcs
+        self._psf = psf
         self._unit = unit
         self._meta = meta
 
@@ -140,8 +153,7 @@ def test_nddata_init_data_ndarray():
 
 def test_nddata_init_data_maskedarray():
     with NumpyRNGContext(456):
-        NDData(np.random.random((10, 10)),
-               mask=np.random.random((10, 10)) > 0.5)
+        NDData(np.random.random((10, 10)), mask=np.random.random((10, 10)) > 0.5)
 
     # Another test (just copied here)
     with NumpyRNGContext(12345):
@@ -165,14 +177,14 @@ def test_nddata_init_data_maskedarray():
     assert nd.data[11] != marr.data[11]
 
 
-@pytest.mark.parametrize('data', [np.array([1, 2, 3]), 5])
+@pytest.mark.parametrize("data", [np.array([1, 2, 3]), 5])
 def test_nddata_init_data_quantity(data):
     # Test an array and a scalar because a scalar Quantity does not always
-    # behaves the same way as an array.
+    # behave the same way as an array.
     quantity = data * u.adu
     ndd = NDData(quantity)
     assert ndd.unit == quantity.unit
-    assert_array_equal(ndd.data, np.array(quantity.value))
+    assert_array_equal(ndd.data, np.array(quantity))
     if ndd.data.size > 1:
         # check that if it is an array it is not copied
         quantity.value[1] = 100
@@ -183,12 +195,17 @@ def test_nddata_init_data_quantity(data):
         quantity.value[1] = 5
         assert ndd.data[1] != quantity.value[1]
 
+    # provide a quantity and override the unit
+    ndd_unit = NDData(data * u.erg, unit=u.J)
+    assert ndd_unit.unit == u.J
+    np.testing.assert_allclose((ndd_unit.data * ndd_unit.unit).to_value(u.erg), data)
+
 
 def test_nddata_init_data_masked_quantity():
     a = np.array([2, 3])
     q = a * u.m
     m = False
-    mq = np.ma.array(q, mask=m)
+    mq = Masked(q, mask=m)
     nd = NDData(mq)
     assert_array_equal(nd.data, a)
     # This test failed before the change in nddata init because the masked
@@ -206,6 +223,7 @@ def test_nddata_init_data_nddata():
     assert nd2.mask == nd1.mask
     assert nd2.unit == nd1.unit
     assert nd2.meta == nd1.meta
+    assert nd2.psf == nd1.psf
 
     # Check that it is copied by reference
     nd1 = NDData(np.ones((5, 5)))
@@ -218,8 +236,15 @@ def test_nddata_init_data_nddata():
     assert nd1.data[2, 3] != nd2.data[2, 3]
 
     # Now let's see what happens if we have all explicitly set
-    nd1 = NDData(np.array([1]), mask=False, uncertainty=StdDevUncertainty(10), unit=u.s,
-                 meta={'dest': 'mordor'}, wcs=WCS(naxis=1))
+    nd1 = NDData(
+        np.array([1]),
+        mask=False,
+        uncertainty=StdDevUncertainty(10),
+        unit=u.s,
+        meta={"dest": "mordor"},
+        wcs=WCS(naxis=1),
+        psf=np.array([10]),
+    )
     nd2 = NDData(nd1)
     assert nd2.data is nd1.data
     assert nd2.wcs is nd1.wcs
@@ -227,30 +252,45 @@ def test_nddata_init_data_nddata():
     assert nd2.mask == nd1.mask
     assert nd2.unit == nd1.unit
     assert nd2.meta == nd1.meta
+    assert nd2.psf == nd1.psf
 
     # now what happens if we overwrite them all too
-    nd3 = NDData(nd1, mask=True, uncertainty=StdDevUncertainty(200), unit=u.km,
-                 meta={'observer': 'ME'}, wcs=WCS(naxis=1))
+    nd3 = NDData(
+        nd1,
+        mask=True,
+        uncertainty=StdDevUncertainty(200),
+        unit=u.km,
+        meta={"observer": "ME"},
+        wcs=WCS(naxis=1),
+        psf=np.array([20]),
+    )
     assert nd3.data is nd1.data
     assert nd3.wcs is not nd1.wcs
     assert nd3.uncertainty.array != nd1.uncertainty.array
     assert nd3.mask != nd1.mask
     assert nd3.unit != nd1.unit
     assert nd3.meta != nd1.meta
+    assert nd3.psf != nd1.psf
 
 
 def test_nddata_init_data_nddata_subclass():
     uncert = StdDevUncertainty(3)
     # There might be some incompatible subclasses of NDData around.
-    bnd = BadNDDataSubclass(False, True, 3, 2, 'gollum', 100)
+    bnd = BadNDDataSubclass(False, True, 3, 2, "gollum", 100, 12)
     # Before changing the NDData init this would not have raised an error but
     # would have lead to a compromised nddata instance
     with pytest.raises(TypeError):
         NDData(bnd)
 
     # but if it has no actual incompatible attributes it passes
-    bnd_good = BadNDDataSubclass(np.array([1, 2]), uncert, 3, HighLevelWCSWrapper(WCS(naxis=1)),
-                                 {'enemy': 'black knight'}, u.km)
+    bnd_good = BadNDDataSubclass(
+        np.array([1, 2]),
+        uncert,
+        3,
+        HighLevelWCSWrapper(WCS(naxis=1)),
+        {"enemy": "black knight"},
+        u.km,
+    )
     nd = NDData(bnd_good)
     assert nd.unit == bnd_good.unit
     assert nd.meta == bnd_good.meta
@@ -263,7 +303,7 @@ def test_nddata_init_data_nddata_subclass():
 def test_nddata_init_data_fail():
     # First one is sliceable but has no shape, so should fail.
     with pytest.raises(TypeError):
-        NDData({'a': 'dict'})
+        NDData({"a": "dict"})
 
     # This has a shape but is not sliceable
     class Shape:
@@ -271,7 +311,7 @@ def test_nddata_init_data_fail():
             self.shape = 5
 
         def __repr__(self):
-            return '7'
+            return "7"
 
     with pytest.raises(TypeError):
         NDData(Shape())
@@ -294,7 +334,7 @@ def test_param_uncertainty():
     # Test that the parent_nddata is set.
     assert d.uncertainty.parent_nddata is d
     # Test conflicting uncertainties (other NDData)
-    u2 = StdDevUncertainty(array=np.ones((5, 5))*2)
+    u2 = StdDevUncertainty(array=np.ones((5, 5)) * 2)
     d2 = NDData(d, uncertainty=u2)
     assert d2.uncertainty is u2
     assert d2.uncertainty.parent_nddata is d2
@@ -316,14 +356,14 @@ def test_param_meta():
     nd = NDData([1, 2, 3], meta={})
     assert len(nd.meta) == 0
     nd = NDData([1, 2, 3])
-    assert isinstance(nd.meta, OrderedDict)
+    assert isinstance(nd.meta, dict)
     assert len(nd.meta) == 0
     # Test conflicting meta (other NDData)
-    nd2 = NDData(nd, meta={'image': 'sun'})
+    nd2 = NDData(nd, meta={"image": "sun"})
     assert len(nd2.meta) == 1
-    nd3 = NDData(nd2, meta={'image': 'moon'})
+    nd3 = NDData(nd2, meta={"image": "moon"})
     assert len(nd3.meta) == 1
-    assert nd3.meta['image'] == 'moon'
+    assert nd3.meta["image"] == "moon"
 
 
 def test_param_mask():
@@ -337,7 +377,7 @@ def test_param_mask():
     nd3 = NDData(np.ma.array([1], mask=False), mask=True)
     assert nd3.mask
     # (masked quantity)
-    mq = np.ma.array(np.array([2, 3])*u.m, mask=False)
+    mq = np.ma.array(np.array([2, 3]) * u.m, mask=False)
     nd4 = NDData(mq, mask=True)
     assert nd4.mask
 
@@ -345,25 +385,29 @@ def test_param_mask():
 def test_param_unit():
     with pytest.raises(ValueError):
         NDData(np.ones((5, 5)), unit="NotAValidUnit")
-    NDData([1, 2, 3], unit='meter')
+    NDData([1, 2, 3], unit="meter")
     # Test conflicting units (quantity as data)
     q = np.array([1, 2, 3]) * u.m
-    nd = NDData(q, unit='cm')
+    nd = NDData(q, unit="cm")
     assert nd.unit != q.unit
     assert nd.unit == u.cm
     # (masked quantity)
-    mq = np.ma.array(np.array([2, 3])*u.m, mask=False)
-    nd2 = NDData(mq, unit=u.s)
-    assert nd2.unit == u.s
+    mq = np.ma.array(np.array([2, 3]) * u.m, mask=False)
+    nd2 = NDData(mq, unit=u.pc)
+    assert nd2.unit == u.pc
     # (another NDData as data)
-    nd3 = NDData(nd, unit='km')
+    nd3 = NDData(nd, unit="km")
     assert nd3.unit == u.km
+    # (MaskedQuantity given to NDData)
+    mq_astropy = Masked.from_unmasked(q, False)
+    nd4 = NDData(mq_astropy, unit="km")
+    assert nd4.unit == u.km
 
 
 def test_pickle_nddata_with_uncertainty():
-    ndd = NDData(np.ones(3),
-                 uncertainty=StdDevUncertainty(np.ones(5), unit=u.m),
-                 unit=u.m)
+    ndd = NDData(
+        np.ones(3), uncertainty=StdDevUncertainty(np.ones(5), unit=u.m), unit=u.m
+    )
     ndd_dumped = pickle.dumps(ndd)
     ndd_restored = pickle.loads(ndd_dumped)
     assert type(ndd_restored.uncertainty) is StdDevUncertainty
@@ -372,13 +416,12 @@ def test_pickle_nddata_with_uncertainty():
 
 
 def test_pickle_uncertainty_only():
-    ndd = NDData(np.ones(3),
-                 uncertainty=StdDevUncertainty(np.ones(5), unit=u.m),
-                 unit=u.m)
+    ndd = NDData(
+        np.ones(3), uncertainty=StdDevUncertainty(np.ones(5), unit=u.m), unit=u.m
+    )
     uncertainty_dumped = pickle.dumps(ndd.uncertainty)
     uncertainty_restored = pickle.loads(uncertainty_dumped)
-    np.testing.assert_array_equal(ndd.uncertainty.array,
-                                  uncertainty_restored.array)
+    np.testing.assert_array_equal(ndd.uncertainty.array, uncertainty_restored.array)
     assert ndd.uncertainty.unit == uncertainty_restored.unit
     # Even though it has a parent there is no one that references the parent
     # after unpickling so the weakref "dies" immediately after unpickling
@@ -396,68 +439,66 @@ def test_pickle_nddata_without_uncertainty():
 # Check that the meta descriptor is working as expected. The MetaBaseTest class
 # takes care of defining all the tests, and we simply have to define the class
 # and any minimal set of args to pass.
-from astropy.utils.tests.test_metadata import MetaBaseTest
-
-
 class TestMetaNDData(MetaBaseTest):
     test_class = NDData
-    args = np.array([[1.]])
+    args = np.array([[1.0]])
 
 
 # Representation tests
 def test_nddata_str():
     arr1d = NDData(np.array([1, 2, 3]))
-    assert str(arr1d) == '[1 2 3]'
+    assert str(arr1d) == "[1 2 3]"
 
     arr2d = NDData(np.array([[1, 2], [3, 4]]))
-    assert str(arr2d) == textwrap.dedent("""
+    assert str(arr2d) == textwrap.dedent(
+        """
         [[1 2]
-         [3 4]]"""[1:])
+         [3 4]]"""[1:]
+    )
 
     arr3d = NDData(np.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]]))
-    assert str(arr3d) == textwrap.dedent("""
+    assert str(arr3d) == textwrap.dedent(
+        """
         [[[1 2]
           [3 4]]
 
          [[5 6]
-          [7 8]]]"""[1:])
+          [7 8]]]"""[1:]
+    )
 
     # let's add units!
     arr = NDData(np.array([1, 2, 3]), unit="km")
-    assert str(arr) == '[1 2 3] km'
+    assert str(arr) == "[1 2 3] km"
 
     # what if it had these units?
     arr = NDData(np.array([1, 2, 3]), unit="erg cm^-2 s^-1 A^-1")
-    assert str(arr) == '[1 2 3] erg / (A cm2 s)'
+    assert str(arr) == "[1 2 3] erg / (A s cm2)"
 
 
 def test_nddata_repr():
     # The big test is eval(repr()) should be equal to the original!
+    # but this must be modified slightly since adopting the
+    # repr machinery from astropy.utils.masked
 
     arr1d = NDData(np.array([1, 2, 3]))
     s = repr(arr1d)
-    assert s == 'NDData([1, 2, 3])'
+    assert s == "NDData([1, 2, 3])"
     got = eval(s)
     assert np.all(got.data == arr1d.data)
     assert got.unit == arr1d.unit
 
     arr2d = NDData(np.array([[1, 2], [3, 4]]))
     s = repr(arr2d)
-    assert s == textwrap.dedent("""
-        NDData([[1, 2],
-                [3, 4]])"""[1:])
+    assert s == ("NDData([[1, 2],\n        [3, 4]])")
     got = eval(s)
     assert np.all(got.data == arr2d.data)
     assert got.unit == arr2d.unit
 
     arr3d = NDData(np.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]]))
     s = repr(arr3d)
-    assert s == textwrap.dedent("""
-        NDData([[[1, 2],
-                 [3, 4]],
-
-                [[5, 6],
-                 [7, 8]]])"""[1:])
+    assert s == (
+        "NDData([[[1, 2],\n         [3, 4]],\n\n        [[5, 6],\n         [7, 8]]])"
+    )
     got = eval(s)
     assert np.all(got.data == arr3d.data)
     assert got.unit == arr3d.unit
@@ -471,6 +512,19 @@ def test_nddata_repr():
     assert got.unit == arr.unit
 
 
+@pytest.mark.skipif(not HAS_DASK, reason="requires dask to be available")
+def test_nddata_repr_dask():
+    import dask.array as da
+
+    arr = NDData(da.arange(3), unit="km")
+    s = repr(arr)
+    # just check repr equality for dask arrays, not round-tripping:
+    assert s in (
+        'NDData(\n  data=dask.array<arange, shape=(3,), dtype=int64, chunksize=(3,), chunktype=numpy.ndarray>,\n  unit=Unit("km")\n)',
+        'NDData(\n  data=dask.array<arange, shape=(3,), dtype=int32, chunksize=(3,), chunktype=numpy.ndarray>,\n  unit=Unit("km")\n)',
+    )
+
+
 # Not supported features
 def test_slicing_not_supported():
     ndd = NDData(np.ones((5, 5)))
@@ -493,18 +547,22 @@ def test_nddata_wcs_setter_error_cases():
 
     naxis = 2
     # This should succeed since the WCS is currently None
-    ndd.wcs = nd_testing._create_wcs_simple(naxis=naxis,
-                                            ctype=['deg'] * naxis,
-                                            crpix=[0] * naxis,
-                                            crval=[10] * naxis,
-                                            cdelt=[1] * naxis)
+    ndd.wcs = nd_testing._create_wcs_simple(
+        naxis=naxis,
+        ctype=["deg"] * naxis,
+        crpix=[0] * naxis,
+        crval=[10] * naxis,
+        cdelt=[1] * naxis,
+    )
     with pytest.raises(ValueError):
         # This should fail since the WCS is not None
-        ndd.wcs = nd_testing._create_wcs_simple(naxis=naxis,
-                                                ctype=['deg'] * naxis,
-                                                crpix=[0] * naxis,
-                                                crval=[10] * naxis,
-                                                cdelt=[1] * naxis)
+        ndd.wcs = nd_testing._create_wcs_simple(
+            naxis=naxis,
+            ctype=["deg"] * naxis,
+            crpix=[0] * naxis,
+            crval=[10] * naxis,
+            cdelt=[1] * naxis,
+        )
 
 
 def test_nddata_wcs_setter_with_low_level_wcs():
@@ -537,3 +595,111 @@ def test_overriden_wcs():
     # Check that a sub-class that overrides `.wcs` without providing a setter
     # works
     NDDataCustomWCS(np.ones((5, 5)))
+
+
+# set up parameters for test_collapse:
+np.random.seed(42)
+collapse_units = [None, u.Jy]
+collapse_propagate = [True, False]
+collapse_data_shapes = [
+    # 3D example:
+    (4, 3, 2),
+    # 5D example
+    (6, 5, 4, 3, 2),
+]
+collapse_ignore_masked = [True, False]
+collapse_masks = list(
+    chain.from_iterable(
+        [
+            # try the operations without a mask (all False):
+            np.zeros(collapse_data_shape).astype(bool)
+        ]
+        + [
+            # assemble a bunch of random masks:
+            np.random.randint(0, 2, size=collapse_data_shape).astype(bool)
+            for _ in range(10)
+        ]
+        for collapse_data_shape in collapse_data_shapes
+    )
+)
+
+# the following provides pytest.mark.parametrize with every
+# permutation of (1) the units, (2) propagating/not propagating
+# uncertainties, and (3) the data shapes of different ndim.
+permute = (
+    len(collapse_masks)
+    * len(collapse_propagate)
+    * len(collapse_units)
+    * len(collapse_ignore_masked)
+)
+collapse_units = permute // len(collapse_units) * collapse_units
+collapse_propagate = permute // len(collapse_propagate) * collapse_propagate
+collapse_masks = permute // len(collapse_masks) * collapse_masks
+collapse_ignore_masked = permute // len(collapse_ignore_masked) * collapse_ignore_masked
+
+
+@pytest.mark.parametrize(
+    "mask, unit, propagate_uncertainties, operation_ignores_mask",
+    zip(collapse_masks, collapse_units, collapse_propagate, collapse_ignore_masked),
+)
+def test_collapse(mask, unit, propagate_uncertainties, operation_ignores_mask):
+    # unique set of combinations of each of the N-1 axes for an N-D cube:
+    axes_permutations = {tuple(axes[:2]) for axes in permutations(range(mask.ndim))}
+
+    # each of the single axis slices:
+    axes_permutations.update(set(range(mask.ndim)))
+    axes_permutations.update({None})
+
+    cube = np.arange(np.prod(mask.shape)).reshape(mask.shape)
+    numpy_cube = np.ma.masked_array(cube, mask=mask)
+    ma_cube = Masked(cube, mask=mask)
+    ndarr = NDDataArray(cube, uncertainty=StdDevUncertainty(cube), unit=unit, mask=mask)
+
+    # By construction, the minimum value along each axis is always the zeroth index and
+    # the maximum is always the last along that axis. We verify that here, so we can
+    # test that the correct uncertainties are extracted during the
+    # `NDDataArray.min` and `NDDataArray.max` methods later:
+    for axis in range(cube.ndim):
+        assert np.all(np.equal(cube.argmin(axis=axis), 0))
+        assert np.all(np.equal(cube.argmax(axis=axis), cube.shape[axis] - 1))
+
+    # confirm that supported nddata methods agree with corresponding numpy methods
+    # for the masked data array:
+    sum_methods = ["sum", "mean"]
+    ext_methods = ["min", "max"]
+    all_methods = sum_methods + ext_methods
+
+    # for all supported methods, ensure the masking is propagated:
+    for method in all_methods:
+        for axes in axes_permutations:
+            astropy_method = getattr(ma_cube, method)(axis=axes)
+            numpy_method = getattr(numpy_cube, method)(axis=axes)
+            nddata_method = getattr(ndarr, method)(
+                axis=axes,
+                propagate_uncertainties=propagate_uncertainties,
+                operation_ignores_mask=operation_ignores_mask,
+            )
+            astropy_unmasked = astropy_method.base[~astropy_method.mask]
+            nddata_unmasked = nddata_method.data[~nddata_method.mask]
+
+            # check if the units are passed through correctly:
+            assert unit == nddata_method.unit
+
+            # check if the numpy and astropy.utils.masked results agree when
+            # the result is not fully masked:
+            if len(astropy_unmasked) > 0:
+                if not operation_ignores_mask:
+                    # compare with astropy
+                    assert np.all(np.equal(astropy_unmasked, nddata_unmasked))
+                    assert np.all(np.equal(astropy_method.mask, nddata_method.mask))
+                else:
+                    # compare with numpy
+                    assert np.ma.all(
+                        np.ma.equal(numpy_method, np.asanyarray(nddata_method))
+                    )
+
+            # For extremum methods, ensure the uncertainty returned corresponds to the
+            # min/max data value. We've created the uncertainties to have the same value
+            # as the data array, so we can just check for equality:
+            if method in ext_methods and propagate_uncertainties:
+                assert np.ma.all(np.ma.equal(astropy_method, nddata_method))
