@@ -9,13 +9,26 @@ celestial-to-terrestrial coordinate transformations
 (in `astropy.coordinates`).
 """
 
+import os
 import re
-from datetime import datetime
+from datetime import datetime, timezone
 from urllib.parse import urlparse
 from warnings import warn
 
 import erfa
 import numpy as np
+from astropy_iers_data import (
+    IERS_A_FILE,
+    IERS_A_README,
+    IERS_A_URL,
+    IERS_A_URL_MIRROR,
+    IERS_B_FILE,
+    IERS_B_README,
+    IERS_B_URL,
+    IERS_LEAP_SECOND_FILE,
+    IERS_LEAP_SECOND_URL,
+)
+from astropy_iers_data import IERS_LEAP_SECOND_URL_MIRROR as IETF_LEAP_SECOND_URL
 
 from astropy import config as _config
 from astropy import units as u
@@ -24,11 +37,10 @@ from astropy.table import MaskedColumn, QTable
 from astropy.time import Time, TimeDelta
 from astropy.utils.data import (
     clear_download_cache,
-    get_pkg_data_filename,
     get_readable_fileobj,
     is_url_in_cache,
 )
-from astropy.utils.exceptions import AstropyWarning
+from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyWarning
 from astropy.utils.state import ScienceState
 
 __all__ = [
@@ -61,22 +73,6 @@ __all__ = [
     "IETF_LEAP_SECOND_URL",
 ]
 
-# IERS-A default file name, URL, and ReadMe with content description
-IERS_A_FILE = "finals2000A.all"
-IERS_A_URL = "https://datacenter.iers.org/data/9/finals2000A.all"
-IERS_A_URL_MIRROR = "https://maia.usno.navy.mil/ser7/finals2000A.all"
-IERS_A_README = get_pkg_data_filename("data/ReadMe.finals2000A")
-
-# IERS-B default file name, URL, and ReadMe with content description
-IERS_B_FILE = get_pkg_data_filename("data/eopc04_IAU2000.62-now")
-IERS_B_URL = "https://hpiers.obspm.fr/iers/eop/eopc04/eopc04_IAU2000.62-now"
-IERS_B_README = get_pkg_data_filename("data/ReadMe.eopc04_IAU2000")
-
-# LEAP SECONDS default file name, URL, and alternative format/URL
-IERS_LEAP_SECOND_FILE = get_pkg_data_filename("data/Leap_Second.dat")
-IERS_LEAP_SECOND_URL = "https://hpiers.obspm.fr/iers/bul/bulc/Leap_Second.dat"
-IETF_LEAP_SECOND_URL = "https://www.ietf.org/timezones/data/leap-seconds.list"
-
 # Status/source values returned by IERS.ut1_utc
 FROM_IERS_B = 0
 FROM_IERS_A = 1
@@ -211,7 +207,7 @@ conf = Conf()
 
 class IERSRangeError(IndexError):
     """
-    Any error for when dates are outside of the valid range for IERS
+    Any error for when dates are outside of the valid range for IERS.
     """
 
 
@@ -506,7 +502,6 @@ class IERS(QTable):
         time values in ``mdj`` and the time span of the table.  The base behavior
         is not to update the table.  ``IERS_Auto`` overrides this method.
         """
-        pass
 
     def ut1_utc_source(self, i):
         """Source for UT1-UTC.  To be overridden by subclass."""
@@ -626,7 +621,25 @@ class IERS_A(IERS):
         ``IERS_A`` class instance
         """
         if file is None:
-            file = IERS_A_FILE
+            # In prior versions of astropy, read() would only work without a
+            # file argument if a finals2000A.all file was present in the
+            # current working directory. We can now use the versions from
+            # astropy-iers-data but for backward-compatibility we first check
+            # if there is a file in the current working directory and use that
+            # if so, emitting a deprecation warning
+            if os.path.exists("finals2000A.all"):
+                file = "finals2000A.all"
+                warn(
+                    "The file= argument was not specified but "
+                    "'finals2000A.all' is present in the current working "
+                    "directory, so reading IERS data from that file. To "
+                    "continue reading a local file from the current working "
+                    "directory, specify file= explicitly otherwise a bundled "
+                    "file will be used in future.",
+                    AstropyDeprecationWarning,
+                )
+            else:
+                file = IERS_A_FILE
         if readme is None:
             readme = IERS_A_README
 
@@ -640,7 +653,7 @@ class IERS_A(IERS):
         return table
 
     def ut1_utc_source(self, i):
-        """Set UT1-UTC source flag for entries in IERS table"""
+        """Set UT1-UTC source flag for entries in IERS table."""
         ut1flag = self["UT1Flag"][i]
         source = np.ones_like(i) * FROM_IERS_B
         source[ut1flag == "I"] = FROM_IERS_A
@@ -648,7 +661,7 @@ class IERS_A(IERS):
         return source
 
     def dcip_source(self, i):
-        """Set CIP correction source flag for entries in IERS table"""
+        """Set CIP correction source flag for entries in IERS table."""
         nutflag = self["NutFlag"][i]
         source = np.ones_like(i) * FROM_IERS_B
         source[nutflag == "I"] = FROM_IERS_A
@@ -656,7 +669,7 @@ class IERS_A(IERS):
         return source
 
     def pm_source(self, i):
-        """Set polar motion source flag for entries in IERS table"""
+        """Set polar motion source flag for entries in IERS table."""
         pmflag = self["PolPMFlag"][i]
         source = np.ones_like(i) * FROM_IERS_B
         source[pmflag == "I"] = FROM_IERS_A
@@ -673,13 +686,16 @@ class IERS_B(IERS):
     -----
     If the package IERS B file (```iers.IERS_B_FILE``) is out of date, a new
     version can be downloaded from ``iers.IERS_B_URL``.
+
+    See `~astropy.utils.iers.IERS_B.read` for instructions on how to read
+    a pre-2023 style IERS B file (usually named ``eopc04_IAU2000.62-now``).
     """
 
     iers_table = None
 
     @classmethod
-    def read(cls, file=None, readme=None, data_start=14):
-        """Read IERS-B table from a eopc04_iau2000.* file provided by IERS.
+    def read(cls, file=None, readme=None, data_start=6):
+        """Read IERS-B table from a eopc04.* file provided by IERS.
 
         Parameters
         ----------
@@ -690,11 +706,29 @@ class IERS_B(IERS):
             full path to ascii file holding CDS-style readme.
             Defaults to package version, ``iers.IERS_B_README``.
         data_start : int
-            starting row. Default is 14, appropriate for standard IERS files.
+            Starting row. Default is 6, appropriate for standard IERS files.
 
         Returns
         -------
         ``IERS_B`` class instance
+
+        Notes
+        -----
+        To read a pre-2023 style IERS B file (usually named something like
+        ``eopc04_IAU2000.62-now``), do something like this example with an
+        excerpt that is used for testing::
+
+            >>> from astropy.utils.iers import IERS_B
+            >>> from astropy.utils.data import get_pkg_data_filename
+            >>> old_style_file = get_pkg_data_filename(
+            ...     "tests/data/iers_b_old_style_excerpt",
+            ...     package="astropy.utils.iers")
+            >>> iers_b = IERS_B.read(
+            ...     old_style_file,
+            ...     readme=get_pkg_data_filename("data/ReadMe.eopc04_IAU2000",
+            ...                                  package="astropy.utils.iers"),
+            ...     data_start=14)
+
         """
         if file is None:
             file = IERS_B_FILE
@@ -708,15 +742,15 @@ class IERS_B(IERS):
         return table
 
     def ut1_utc_source(self, i):
-        """Set UT1-UTC source flag for entries in IERS table"""
+        """Set UT1-UTC source flag for entries in IERS table."""
         return np.ones_like(i) * FROM_IERS_B
 
     def dcip_source(self, i):
-        """Set CIP correction source flag for entries in IERS table"""
+        """Set CIP correction source flag for entries in IERS table."""
         return np.ones_like(i) * FROM_IERS_B
 
     def pm_source(self, i):
-        """Set PM source flag for entries in IERS table"""
+        """Set PM source flag for entries in IERS table."""
         return np.ones_like(i) * FROM_IERS_B
 
 
@@ -1056,7 +1090,9 @@ class LeapSeconds(QTable):
     def _today():
         # Get current day in scale='tai' without going through a scale change
         # (so we do not need leap seconds).
-        s = "{0.year:04d}-{0.month:02d}-{0.day:02d}".format(datetime.utcnow())
+        s = "{0.year:04d}-{0.month:02d}-{0.day:02d}".format(
+            datetime.now(tz=timezone.utc)
+        )
         return Time(s, scale="tai", format="iso", out_subfmt="date")
 
     @classmethod
@@ -1150,7 +1186,7 @@ class LeapSeconds(QTable):
 
     @classmethod
     def _read_leap_seconds(cls, file, **kwargs):
-        """Read a file, identifying expiration by matching 'File expires'"""
+        """Read a file, identifying expiration by matching 'File expires'."""
         expires = None
         # Find expiration date.
         with get_readable_fileobj(file) as fh:
@@ -1239,8 +1275,9 @@ class LeapSeconds(QTable):
             with erfa.
         """
         current = cls(erfa.leap_seconds.get())
+        expires = erfa.leap_seconds.expires
         current._expires = Time(
-            "{0.year:04d}-{0.month:02d}-{0.day:02d}".format(erfa.leap_seconds.expires),
+            f"{expires.year:04d}-{expires.month:02d}-{expires.day:02d}",
             scale="tai",
         )
         if not built_in:
